# -*- coding: utf8 -*-
from __future__ import unicode_literals

from collections import OrderedDict
import datetime

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimList,
    HprimMultiple,
    HprimString,
    Segment,
)


class IhcomSegmentH(HeaderSegment):
    sender_id = HprimString(5)
    sender_address = HprimString(6)
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class IhcomSegmentP(Segment):
    patient_id = HprimString(3)
    laboratory_patient_id = HprimString(4)
    patient_name = HprimString((6, 1))
    patient_first_name = HprimString((6, 2))
    maiden_name = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    hospital_service1 = HprimString((33, 1))

    orders = HprimForeign("O", "patient")


class IhcomSegmentO(Segment):
    specimen_id = HprimString(3)
    specimen_code = HprimString((4, 1))

    # Types de prélèvements possibles:
    # CENTBLOOD, PACKED-CELLS, PLASMA, 0.8CELLS, 3CELLS
    specimen_type = HprimString((4, 2))
    test_id = HprimString((5, 4))
    batch_id = HprimString((5, 6))
    test_code = HprimString((5, 7))
    priority = HprimString(6)
    request_ordered = HprimDate(7)
    collection_date = HprimDate(8)
    action_code = HprimString(12)
    user_field_no1 = HprimString(19)
    user_field_no2 = HprimString(20)
    results_reported_date = HprimDate(23)
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class IhcomSegmentR(Segment):
    manufacturer_test_type = HprimString((3, 4))
    manufacturer_test_name = HprimString((3, 5))
    manufacturer_test_code = HprimString((3, 6))
    test_code = HprimString((3, 7))
    # Result code (End result)
    value = HprimString((4, 1))
    # Result code of first reading
    initial_value = HprimString((4, 2))
    # Result code of the automate
    device_value = HprimString((4, 3))
    pheno_result = HprimString((4, 4))
    kell_result = HprimString((4, 5))
    dat_result = HprimString((4, 6))
    xmatch_result = HprimString((4, 7))
    autocontrol = HprimString((4, 8))
    other_antigen = HprimString((4, 9))
    unit = HprimString(5)
    reference_range = HprimString(6)
    result_status = HprimString(9)
    operator_id_performed = HprimString((11, 1))
    operator_id_verified = HprimString((11, 2))
    date_test_started = HprimDate(12)
    date_test_completed = HprimDate(13)
    instrument_id = HprimString(14)


class IhcomSegmentC(Segment):
    comment_text = HprimList(
        3,
        basefield=HprimMultiple(
            0,
            description=HprimString(1),
            lot_number=HprimString(2),
            cassette_lot=HprimString(3),
            expiration_date=HprimString(4),
        ),
    )
    test_media_type = HprimString((4, 1))
    barcode_plate = HprimString((4, 2))
    cassette_lot_number = HprimString((4, 3))
    expiration_date_plate = HprimDate((4, 4))
    well_plate_number = HprimString((4, 5))
    comment = HprimString(5)
    # Optional, not available if using socket protocol or serial protocol
    filename_reaction_picture = HprimString(6)


class IhcomSegmentL(Segment):
    terminaison_code = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {"code": "ihcom_astm", "label": "IH COM ASTM Bio-Rad"}
    SEGMENT_CLASS_MAP = {
        "H": IhcomSegmentH,
        "P": IhcomSegmentP,
        "O": IhcomSegmentO,
        "R": IhcomSegmentR,
        "C": IhcomSegmentC,
        "L": IhcomSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"
    OPTIONS = [
        AstmMessageV2.BooleanOption("ihcom_resu_precedent_comment", "Mettre les résultats précédents en commentaire", default=True),
    ]

    def __init__(self, **kwargs):
        self.resu_precedent_comment = kwargs.pop('ihcom_resu_precedent_comment', True)
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = IhcomSegmentH(type_segment="H", **kwargs)
        message.end = IhcomSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
            action={"del": "C", "add": "N", "new": "N"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                laboratory_patient_id=patient.lab_patient_id,
                patient_name=patient.last_name,
                patient_first_name=patient.first_name,
                maiden_name=patient.maiden_name,
                birthdate=patient.birthdate,
                sex=patient.sex or "U",
            )
            for sample, action, priority in patient.samples:
                for order in sample.orders:
                    seg_p.orders.add(
                        specimen_code=sample.sample_id,
                        specimen_type=sample.specimen,
                        test_id=order["code"],
                        priority=priority,
                        request_ordered=sample.request_date,
                        collection_date=sample.collection_date,
                        action_code=action,
                    )

        if msg.patients:
            return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                for seg_r in seg_o.results:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.specimen_code, type="patient"
                    )
                    # Résultat du phénotype éclaté
                    # Résultat du groupe et du rhésus éclaté
                    code = "{}_{}".format(
                        seg_r.manufacturer_test_name, seg_r.manufacturer_test_type
                    )
                    result = None
                    if (
                        seg_r.manufacturer_test_type != "Result" and
                        seg_r.value and
                        (seg_r.result_status == 'M' or not seg_r.initial_value) and
                        not seg_r.pheno_result and
                        not seg_r.kell_result
                    ):
                        value = seg_r.value
                        comments = []
                        if self.resu_precedent_comment:
                            if seg_r.initial_value:
                                comments.append("Initial value = {}".format(seg_r.initial_value))
                            if seg_r.device_value:
                                comments.append("Device value = {}".format(seg_r.device_value))
                        for seg_c in seg_r.commentaires:
                            if seg_c.comment and not seg_c.comment_text:
                                comments.append(seg_c.comment)
                        result = sample.results.add(
                            code=code,
                            value=value,
                            flags=[seg_r.result_status] if seg_r.result_status != 'R' else [],
                            date=seg_r.date_test_completed,
                            instrument=seg_r.instrument_id,
                            operator=seg_r.operator_id_verified,
                            comments=comments,
                        )

                    # Traitement des résultats de type "Result"
                    liste_resultats = []
                    if seg_r.manufacturer_test_type == "Result":
                        for label, value in OrderedDict(
                            (
                                ("GR", seg_r.value),
                                ("RH", seg_r.initial_value),
                                (
                                    "GRRH",
                                    "{}{}".format(
                                        seg_r.value or "", seg_r.initial_value or ""
                                    ),
                                ),
                                ("ABSsc", seg_r.device_value),
                                ("PHENO", seg_r.pheno_result),
                                ("K", seg_r.kell_result),
                                ("DAT", seg_r.dat_result),
                                ("XMATCH", seg_r.xmatch_result),
                                ("AUTOCONTROL", seg_r.autocontrol),
                                ("OTHER_ANTIGEN", seg_r.other_antigen),
                            )
                        ).items():
                            if not value:
                                continue
                            liste_resultats.append((label, value))
                        for elt in liste_resultats:
                            result = sample.results.add(
                                code="{}_{}_{}".format(
                                    seg_r.manufacturer_test_name,
                                    seg_r.manufacturer_test_type,
                                    elt[0],
                                ),
                                value=elt[1],
                                date=seg_r.date_test_completed,
                                instrument=seg_r.instrument_id,
                                operator=seg_r.operator_id_verified,
                            )
                    if result:
                        for seg_c in seg_r.commentaires:
                            for comment_text in seg_c.comment_text:
                                if "ID-Diluent" in comment_text.description:
                                    # Reagent detected
                                    result.lots.add(
                                        type="reagent",
                                        lot=comment_text.cassette_lot,
                                        expiration=datetime.datetime.strptime(
                                            comment_text.expiration_date, "%Y%m%d"
                                        ),
                                    )

        return worklist
