# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    Segment,
    HeaderSegment,
    HprimString as HL7String,
    HprimDate as HL7Date,
    HprimForeign as HL7Foreign,
)


class UrgentecSegmentMSH(HeaderSegment):
    sending_application = HL7String(3)
    date = HL7Date(7)
    message_type = HL7String((9, 1))
    message_code = HL7String((9, 2))
    processing_id = HL7String(11)
    version = HL7String(12)
    accept_acknowledgment_type = HL7String(15)
    application_acknowledgment_type = HL7String(16)

    orders = HL7Foreign("OBR", "header")


class UrgentecSegmentOBR(Segment):
    specimen_id = HL7String(3)
    test_id = HL7String(5)
    date = HL7Date(8)
    technician = HL7String(34)

    results = HL7Foreign("OBX", "order")
    infos = HL7Foreign("NTE", "order")


class UrgentecSegmentOBX(Segment):
    type_message = HL7String(3)
    test_id = HL7String(4)
    value = HL7String(6)

    infos = HL7Foreign("NTE", "order")


class UrgentecSegmentNTE(Segment):
    type_message = HL7String(3)
    comment = HL7String(4)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "urgentec",
        "label": "URGENTEC HL7 FastFinder",
    }
    SEGMENT_CLASS_MAP = {
        "MSH": UrgentecSegmentMSH,
        "OBR": UrgentecSegmentOBR,
        "OBX": UrgentecSegmentOBX,
        "NTE": UrgentecSegmentNTE,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    def create(self, **kwargs):
        return UrgentecSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_obr in message.orders:
            sample = worklist.get_or_create(
                sample_id=seg_obr.specimen_id,
                type="patient"
            )
            for seg_obx in seg_obr.results:
                flags = []
                result = sample.results.add(
                    code=seg_obx.test_id,
                    value=seg_obx.value,
                    date=seg_obr.date,
                )
                if seg_obr.technician:
                    result.operator = seg_obr.technician
                for seg_nte in seg_obx.infos:
                    if seg_nte.comment:
                        flags.append(seg_nte.comment)
                        result.flags = flags

        return worklist
